==> pyvisual-0.20.tar.gz: pyvisual-0.20/pyvisual/ui/inputs/pv_text_input.py <==
from kivy.uix.widget import Widget
from kivy.uix.textinput import TextInput
from kivy.graphics import Color, Line, RoundedRectangle


class PvTextInput(Widget):
    def __init__(self, container, x, y, width=200, height=40, background_color=(1, 1, 1, 1), input_type="text", visibility=True,
                 placeholder="Enter your text here...", default_text="",
                 text_padding_left=10, text_padding_right=10, text_padding_top=0, text_padding_bottom=10,
                 font="Roboto", font_size=None, font_color=(0.3, 0.3, 0.3, 1),
                 border_color=(0.6, 0.6, 0.6, 1), border_thickness=1, border_style=("bottom", "top", "right", "left"),
                 on_input=None, tag=None, disabled=False, disabled_opacity=0.5,multiline=False,corner_radius=5):
        super().__init__()

        self.corner_radius = corner_radius

        # Store padding values and border style
        self.text_padding_left = text_padding_left
        self.text_padding_right = text_padding_right
        self.text_padding_top = text_padding_top
        self.text_padding_bottom = text_padding_bottom
        self.border_style = border_style
        self.tag = tag
        self.disabled = disabled  # Initialize disabled state
        self.disabled_opacity = disabled_opacity
        self.multiline = multiline
        self.on_input = on_input

        # Calculate the font size based on the height if not specified
        if not font_size:
            # Calculate the available height by subtracting top and bottom padding
            available_height = height - text_padding_top - text_padding_bottom
            self.font_size = available_height * 0.6  # Use 60% of the available height for font size
        else:
            self.font_size = font_size

        # Set widget properties
        self.size_hint = (None, None)
        self.size = (width, height)
        self.pos = (x, y)

        # Create the TextInput as an internal widget with placeholder and default text
        self.text_input = TextInput(
            size_hint=(None, None),
            size=(width - border_thickness * 2, height - border_thickness * 2),
            pos=(x + border_thickness, y + border_thickness),
            font_size=self.font_size,
            font_name=font,
            foreground_color=font_color,
            cursor_color=(0, 0, 0, 1),
            background_normal='',
            background_active='',
            background_color=(0, 0, 0, 0),
            multiline=self.multiline,
            hint_text=placeholder,
            hint_text_color=(0.7, 0.7, 0.7, 1),
            text=default_text,
            password=input_type == "password",
            padding=[self.text_padding_left, self.text_padding_bottom, self.text_padding_right, self.text_padding_top],
            disabled=disabled
        )

        # Apply input restrictions based on the `input_type`
        self.apply_input_restrictions(input_type)

        # If a text callback is provided, bind it
        if self.on_input:
            self.text_input.bind(text=self.on_input)

        # Draw the background of the text box using canvas
        # Draw the background of the text box using canvas
        with self.canvas:
            Color(*background_color)
            self.rect = RoundedRectangle(pos=self.pos, size=self.size, radius=[self.corner_radius])

        # Draw the border or lines based on the `border_style`
        self.update_border_style(border_color, border_thickness)

        # Add the internal TextInput to the parent widget (BasicTextInput)


        # Update the border and rectangle on position and size change
        self.bind(pos=self.update_graphics, size=self.update_graphics)
        self.add_widget(self.text_input)
        # Add the BasicTextInput to the main container
        if container:
            container.add_widget(self)

        self.visibility = visibility
        self.set_visibility(self.visibility)
        self.set_disabled(self.disabled)

    def apply_input_restrictions(self, input_type):
        """Apply input restrictions based on the input type."""
        if input_type == "number":
            self.text_input.input_filter = 'int'
        elif input_type == "float":
            self.text_input.input_filter = 'float'
        elif input_type == "email":
            self.text_input.input_filter = lambda text, from_undo: text.replace(" ", "")
        elif input_type == "alphabet":
            self.text_input.input_filter = lambda text, from_undo: "".join([c for c in text if c.isalpha()])
        else:
            self.text_input.input_filter = None

    def update_border_style(self, border_color, border_thickness):
        """Draw the borders based on the specified style."""
        with self.canvas:
            Color(*border_color)
            # Check if all border styles are present for rounded corners
            if {"top", "bottom", "left", "right"}.issubset(self.border_style) and self.corner_radius > 0:
                # Draw rounded borders
                self.top_line = Line(
                    rounded_rectangle=(self.x, self.y, self.width, self.height, self.corner_radius),
                    width=border_thickness
                )
            else:
                # Draw normal straight borders
                if "top" in self.border_style:
                    self.top_line = Line(points=[self.x, self.top, self.x + self.width, self.top],
                                         width=border_thickness)
                if "bottom" in self.border_style:
                    self.bottom_line = Line(points=[self.x, self.y, self.x + self.width, self.y],
                                            width=border_thickness)
                if "left" in self.border_style:
                    self.left_line = Line(points=[self.x, self.y, self.x, self.top], width=border_thickness)
                if "right" in self.border_style:
                    self.right_line = Line(points=[self.right, self.y, self.right, self.top], width=border_thickness)

    def update_graphics(self, *args):
        """Update the canvas rectangle and borders when the widget size or position changes."""
        # Update background rectangle
        self.rect.pos = self.pos
        self.rect.size = self.size

        # Check if all border styles are present for rounded corners
        if {"top", "bottom", "left", "right"}.issubset(self.border_style) and self.corner_radius > 0:
            self.rect.radius = [self.corner_radius]
            if hasattr(self, 'top_line'):
                self.top_line.rounded_rectangle = (self.x, self.y, self.width, self.height, self.corner_radius)
        else:
            self.rect.radius = [0]  # No rounded corners
            if hasattr(self, 'top_line'):
                self.top_line.points = [self.x, self.top, self.x + self.width, self.top]
            if hasattr(self, 'bottom_line'):
                self.bottom_line.points = [self.x, self.y, self.x + self.width, self.y]
            if hasattr(self, 'left_line'):
                self.left_line.points = [self.x, self.y, self.x, self.top]
            if hasattr(self, 'right_line'):
                self.right_line.points = [self.right, self.y, self.right, self.top]

    def get_text(self):
        """Get the current text value from the internal TextInput."""
        return self.text_input.text

    def set_text(self, value):
        """Set the text value of the internal TextInput."""
        self.text_input.text = value
        # if self.on_input:
        #     self.on_input(self)

    # def set_on_input(self,callback):
    #     self.set_on_input(callback)

    def set_font_size(self,font_size):
        self.text_input.font_size=font_size

    def set_corner_radius(self, corner_radius):
        """Set the corner radius and update the appearance."""
        self.corner_radius = corner_radius
        self.update_graphics()

    def set_border_style(self, border_style):
        """Set the border style and update the graphics."""
        self.border_style = border_style
        self.update_border_style(self.border_color, self.border_thickness)
        self.update_graphics()

    def get_font_size(self):
        return self.text_input.font_size

    def set_cursor_position(self, position):
        """Set the cursor position to the specified index in the text."""
        if 0 <= position <= len(self.text_input.text):
            self.text_input.cursor = (position, 0)  # (column, row)
        else:
            raise ValueError(
                f"Cursor position {position} is out of range for the current text length {len(self.text_input.text)}.")

    def set_visibility(self, visibility):
        """Show or hide the line."""
        if visibility:
            self.opacity = self.disabled_opacity if self.disabled else 1
            self.canvas.opacity = self.opacity
        else:
            self.opacity = 0
            self.canvas.opacity = 0

        self.visibility = visibility

    def set_disabled(self, disabled):
        """Enable or disable the text input."""
        self.disabled = disabled
        self.text_input.disabled = disabled
        self.opacity = self.disabled_opacity if self.disabled else 1


# Example usage of the BasicTextInput class
if __name__ == "__main__":
    import pyvisual as pv
    window = pv.PvWindow()

    # Create an instance of BasicTextInput with adjusted settings
    text_input_default = PvTextInput(
        container=window,
        x=100, y=300,  # Position
        width=400,
        height=50,  # Adjust height
        disabled=False,
        corner_radius=0,
        background_color=(0.9,0.9,0.9,1),
        border_thickness=2,
        border_style=("bottom")
    )

    # Display the window
    window.show()

==> pyvisual-0.23.tar.gz: pyvisual-0.23/pyvisual/ui/inputs/pv_text_input.py <==
from kivy.uix.widget import Widget
from kivy.uix.textinput import TextInput
from kivy.graphics import Color, Line, RoundedRectangle
from kivy.core.text import Label as CoreLabel


class PvTextInput(Widget):
    def __init__(self, container, x=50, y=50, width=200, height=40, background_color=(1, 1, 1, 1), input_type="text", visibility=True,
                 placeholder="Enter your text here...", default_text="",
                 text_padding_left=10, text_padding_right=10, text_padding_top=0, text_padding_bottom=10,
                 font="Roboto", font_size=None, font_color=(0.3, 0.3, 0.3, 1),
                 border_color=(0.6, 0.6, 0.6, 1), border_thickness=1, border_style=("bottom", "top", "right", "left"),
                 on_input=None, tag=None, disabled=False, disabled_opacity=0.5,multiline=False,corner_radius=5):
        super().__init__()

        self.corner_radius = corner_radius

        # Store padding values and border style
        self.text_padding_left = text_padding_left
        self.text_padding_right = text_padding_right
        self.text_padding_top = text_padding_top
        self.text_padding_bottom = text_padding_bottom
        self.border_style = border_style
        self.tag = tag
        self.disabled = disabled  # Initialize disabled state
        self.disabled_opacity = disabled_opacity
        self.multiline = multiline
        self.on_input = on_input

        # Calculate the font size based on the height if not specified
        if not font_size:
            # Calculate the available height by subtracting top and bottom padding
            available_height = height - text_padding_top - text_padding_bottom
            self.font_size = available_height * 0.6  # Use 60% of the available height for font size
        else:
            self.font_size = font_size

        # Set widget properties
        self.size_hint = (None, None)
        self.size = (width, height)
        self.pos = (x, y)

        self.border_thickness = border_thickness

        # Create the TextInput as an internal widget with placeholder and default text
        self.text_input = TextInput(
            size_hint=(None, None),
            size=(width - border_thickness * 2, height - border_thickness * 2),
            pos=(x + border_thickness, y + border_thickness),
            font_size=self.font_size,
            font_name=font,
            foreground_color=font_color,
            cursor_color=(0, 0, 0, 1),
            background_normal='',
            background_active='',
            background_color=(0, 0, 0, 0),
            multiline=self.multiline,
            hint_text=placeholder,
            hint_text_color=(0.7, 0.7, 0.7, 1),
            text=default_text,
            password=input_type == "password",
            padding=[self.text_padding_left, self.text_padding_bottom, self.text_padding_right, self.text_padding_top],
            disabled=disabled
        )

        # Apply input restrictions based on the `input_type`
        self.apply_input_restrictions(input_type)

        # If a text callback is provided, bind it
        if self.on_input:
            self.text_input.bind(text=self.on_input)

        # Draw the background of the text box using canvas
        # Draw the background of the text box using canvas
        with self.canvas:
            Color(*background_color)
            self.rect = RoundedRectangle(pos=self.pos, size=self.size, radius=[self.corner_radius])

        # Draw the border or lines based on the `border_style`
        self.update_border_style(border_color, border_thickness)

        # Add the internal TextInput to the parent widget (BasicTextInput)


        # Update the border and rectangle on position and size change
        self.bind(pos=self.update_graphics, size=self.update_graphics)
        self.add_widget(self.text_input)
        # Add the BasicTextInput to the main container
        if container:
            container.add_widget(self)

        self.visibility = visibility
        self.set_visibility(self.visibility)
        self.set_disabled(self.disabled)

    def apply_input_restrictions(self, input_type):
        """Apply input restrictions based on the input type."""
        if input_type == "number":
            self.text_input.input_filter = 'int'
        elif input_type == "float":
            self.text_input.input_filter = 'float'
        elif input_type == "email":
            self.text_input.input_filter = lambda text, from_undo: text.replace(" ", "")
        elif input_type == "alphabet":
            self.text_input.input_filter = lambda text, from_undo: "".join([c for c in text if c.isalpha()])
        else:
            self.text_input.input_filter = None

    def update_border_style(self, border_color, border_thickness):
        """Draw the borders based on the specified style."""
        with self.canvas:
            Color(*border_color)
            # Check if all border styles are present for rounded corners
            if {"top", "bottom", "left", "right"}.issubset(self.border_style) and self.corner_radius > 0:
                # Draw rounded borders
                self.top_line = Line(
                    rounded_rectangle=(self.x, self.y, self.width, self.height, self.corner_radius),
                    width=border_thickness
                )
            else:
                # Draw normal straight borders
                if "top" in self.border_style:
                    self.top_line = Line(points=[self.x, self.top, self.x + self.width, self.top],
                                         width=border_thickness)
                if "bottom" in self.border_style:
                    self.bottom_line = Line(points=[self.x, self.y, self.x + self.width, self.y],
                                            width=border_thickness)
                if "left" in self.border_style:
                    self.left_line = Line(points=[self.x, self.y, self.x, self.top], width=border_thickness)
                if "right" in self.border_style:
                    self.right_line = Line(points=[self.right, self.y, self.right, self.top], width=border_thickness)

    def update_graphics(self, *args):
        """Update the canvas rectangle, borders, and TextInput position when the widget size or position changes."""
        # Update background rectangle
        self.rect.pos = self.pos
        self.rect.size = self.size

        # Update internal TextInput position
        self.text_input.pos = (
            self.pos[0] + self.border_thickness,
            self.pos[1] + self.border_thickness
        )
        self.text_input.size = (
            self.size[0] - 2 * self.border_thickness,
            self.size[1] - 2 * self.border_thickness
        )

        # Update borders
        if {"top", "bottom", "left", "right"}.issubset(self.border_style) and self.corner_radius > 0:
            self.rect.radius = [self.corner_radius]
            if hasattr(self, 'top_line'):
                self.top_line.rounded_rectangle = (self.x, self.y, self.width, self.height, self.corner_radius)
        else:
            self.rect.radius = [0]  # No rounded corners
            if hasattr(self, 'top_line'):
                self.top_line.points = [self.x, self.top, self.x + self.width, self.top]
            if hasattr(self, 'bottom_line'):
                self.bottom_line.points = [self.x, self.y, self.x + self.width, self.y]
            if hasattr(self, 'left_line'):
                self.left_line.points = [self.x, self.y, self.x, self.top]
            if hasattr(self, 'right_line'):
                self.right_line.points = [self.right, self.y, self.right, self.top]

    def get_text(self):
        """Get the current text value from the internal TextInput."""
        return self.text_input.text

    def set_text(self, value):
        """Set the text value of the internal TextInput."""
        self.text_input.text = value
        # if self.on_input:
        #     self.on_input(self)

    # def set_on_input(self,callback):
    #     self.set_on_input(callback)

    def set_font_size(self,font_size):
        self.text_input.font_size=font_size

    def set_corner_radius(self, corner_radius):
        """Set the corner radius and update the appearance."""
        self.corner_radius = corner_radius
        self.update_graphics()

    def set_border_style(self, border_style):
        """Set the border style and update the graphics."""
        self.border_style = border_style
        self.update_border_style(self.border_color, self.border_thickness)
        self.update_graphics()



    def get_font_size(self):
        return self.text_input.font_size


    def get_text_dimensions(self):
        """Get the width and height of the text currently inside the TextInput."""
        # Use the existing text, font size, and font name
        text = self.text_input.text or " "  # Ensure at least one space for calculation
        font_size = self.text_input.font_size
        font_name = self.text_input.font_name

        # Create a CoreLabel instance to measure the text
        label = CoreLabel(
            text=text,
            font_size=font_size,
            font_name=font_name
        )
        label.refresh()  # Calculate the size
        return label.texture.size  # Return the (width, height) of the text texture

    def set_cursor_position(self, position):
        """Set the cursor position to the specified index in the text."""
        if 0 <= position <= len(self.text_input.text):
            self.text_input.cursor = (position, 0)  # (column, row)
        else:
            raise ValueError(
                f"Cursor position {position} is out of range for the current text length {len(self.text_input.text)}.")

    def set_visibility(self, visibility):
        """Show or hide the line."""
        if visibility:
            self.opacity = self.disabled_opacity if self.disabled else 1
            self.canvas.opacity = self.opacity
        else:
            self.opacity = 0
            self.canvas.opacity = 0

        self.visibility = visibility

    def set_disabled(self, disabled):
        """Enable or disable the text input."""
        self.disabled = disabled
        self.text_input.disabled = disabled
        self.opacity = self.disabled_opacity if self.disabled else 1


# Example usage of the BasicTextInput class
if __name__ == "__main__":
    import pyvisual as pv
    window = pv.PvWindow()

    # Create an instance of BasicTextInput with adjusted settings
    text_input_default = PvTextInput(
        container=window,
        x=100, y=300,  # Position
        width=400,
        height=50,  # Adjust height
        disabled=False,
        corner_radius=0,
        background_color=(0.9,0.9,0.9,1),
        border_thickness=2,
        border_style=("bottom")
    )

    # Display the window
    window.show()
